/********************************************************
 *  WINBOND W569 MIDDLEWARE PLAY ENGINE INTERFACE
 *
 *  FILENAME:
 *    PlayEgn.c
 *
 *  MODIFICATION HISTORY:
 *    2004/08/02	created		by	CCChien0
 ********************************************************/
#include <stdio.h>
#include "TTypeLt.h"
#include "Define.h"
#include "MWDefine.h"
#include "PlayEgn.h"

#if	!USE_SERIAL_INTERFACE

extern void   CPUHost_WriteW569Command( BYTE byData );
extern BYTE   CPUHost_ReadW569Status( void );
extern void   CPUHost_WriteW569Data( const BYTE byData );
extern BYTE   CPUHost_ReadW569Data( void );

#define	WRITE_W569_DATA(CMD, DATA)	\
			{	\
				CPUHost_WriteW569Command(CMD);	\
				CPUHost_WriteW569Data(DATA);	\
			}
#define	WRITE_W569_DATA_AND(CMD, DATA, AND_DATA)	\
			{	\
				CPUHost_WriteW569Command(CMD);	\
				DATA = CPUHost_ReadW569Data();	\
				DATA &= (AND_DATA);				\
				CPUHost_WriteW569Data(DATA);	\
			}
#define	WRITE_W569_DATA_OR(CMD, DATA, OR_DATA)	\
			{	\
				CPUHost_WriteW569Command(CMD);	\
				DATA = CPUHost_ReadW569Data();	\
				DATA |= (OR_DATA);		\
				CPUHost_WriteW569Data(DATA);	\
			}
#define	WRITE_W569_DATA_AND_OR(CMD, DATA, AND_DATA, OR_DATA)	\
			{	\
				CPUHost_WriteW569Command(CMD);	\
				DATA = CPUHost_ReadW569Data();	\
				DATA &= (AND_DATA);		\
				DATA |= (OR_DATA);		\
				CPUHost_WriteW569Data(DATA);	\
			}
#define	WRITE_W569_BLOCK_DATA(CMD, DATA_PTR, DATA_LEN)	\
			{	\
				CPUHost_WriteW569Command(CMD);	\
				while(DATA_LEN > 0)		\
				{	\
					CPUHost_WriteW569Data(*DATA_PTR++);	\
					DATA_LEN--;		\
				}	\
			}
#define	READ_W569_DATA(CMD, DATA)	\
			{	\
				CPUHost_WriteW569Command(CMD);	\
				DATA = CPUHost_ReadW569Data();	\
			}

#else

extern void   CPUHost_SerialWriteW569Data( BYTE byCommand, BYTE byData );
extern BYTE   CPUHost_SerialReadW569Data( BYTE byCommand );
extern void   CPUHost_SerialWriteW569BlockDate( BYTE byCommand,
    const BYTE * pbyData, WORD wDataLen );

#define	WRITE_W569_DATA(CMD, DATA)	\
			{	\
				CPUHost_SerialWriteW569Data(CMD, DATA);	\
			}
#define	WRITE_W569_DATA_AND(CMD, DATA, AND_DATA)	\
			{	\
				DATA = CPUHost_SerialReadW569Data(CMD);	\
				DATA &= (AND_DATA);						\
				CPUHost_SerialWriteW569Data(CMD, DATA);	\
			}
#define	WRITE_W569_DATA_OR(CMD, DATA, OR_DATA)	\
			{	\
				DATA = CPUHost_SerialReadW569Data(CMD);	\
				DATA |= (OR_DATA);						\
				CPUHost_SerialWriteW569Data(CMD, DATA);	\
			}
#define	WRITE_W569_DATA_AND_OR(CMD, DATA, AND_DATA, OR_DATA)	\
			{	\
				DATA = CPUHost_SerialReadW569Data(CMD);	\
				DATA &= (AND_DATA);						\
				DATA |= (OR_DATA);						\
				CPUHost_SerialWriteW569Data(CMD, DATA);	\
			}
#define	WRITE_W569_BLOCK_DATA(CMD, DATA_PTR, DATA_LEN) \
			{	\
				CPUHost_SerialWriteW569BlockDate(CMD, DATA_PTR, DATA_LEN);	\
			}
#define	READ_W569_DATA(CMD, DATA)	\
			{	\
				DATA = CPUHost_SerialReadW569Data(CMD);	\
			}

static BYTE
CPUHost_ReadW569Status( void )
{
  BYTE		byData;

  READ_W569_DATA( CMD_COM_ID_ST_REG, byData );
  return byData;
}

#endif

#define	STATUS_STOPPED		0
#define	STATUS_PLAYING		1
#define	STATUS_PAUSED		2
#define	STATUS_SPEECH_PLAYING	0x80
#define	STATUS_MELODY_PLAYING	0x40

static const BYTE *g_pbyMelodyData;
static DWORD  g_dwMelodyDataSize;
static const BYTE *g_pbyMelodyDataPtr;
static DWORD  g_dwRestMelodyDataSize;
static BYTE   g_byRepeatNum;
static BYTE   g_byRepeatCount;
static BYTE   g_byPlayStatus;
static BOOL   g_bSendTail;
static BYTE   g_bySpeakerVolume;
static BYTE   g_byEQVol;
static BYTE   g_byFormat;

#if	SUPPORT_SPEECH
static BYTE   g_bySpeechPlayStatus;
static BYTE   g_bySpeechType;
static BYTE   g_bySpeechRepeatNum;
static BYTE   g_bySpeechRepeatCount;
static const BYTE *g_pbyAudioData;
static DWORD  g_dwAudioDataSize;
static const BYTE *g_pbyAudioDataPtr;
static DWORD  g_dwRestAudioDataSize;
static BYTE   g_bySpeechTailSize;
#endif

#if (W56940_64 && (DOUT_LED == 0))
static BYTE   g_byHPLeftVol;
static BYTE   g_byHPRightVol;
#endif

// DevDrv
static BOOL   g_bPowerUp;
//BYTE		NRPN[4];
//ComDrv
static BYTE   g_byInterruptMask;
static BYTE   g_bySpeakerVolume;
static WORD   g_wWaitTime;

#if	SUPPORT_LED_CONTROL
static S_LEDSetting g_LEDSetting;
#endif
#if	SUPPORT_MTR_CONTROL
static S_MotorSetting g_MotorSetting;
#endif

#define	LOOP_CHECK_COUNT	10000

#ifdef	FPGA
#define	NO_TIME_OUT		1
#else
#define	NO_TIME_OUT		0
#endif

/*--------------------------------------------------------------------------*/
void
ComDrv_DisableInterrupt( void )
{
  if ( g_byInterruptMask == 1 )
  {
#if	!USE_SERIAL_INTERFACE
    CPUHost_WriteW569Command( 0x00 );
#else
    CPUHost_SerialReadW569Data( CMD_COM_ID_ST_REG );
#endif
  }
}

/*--------------------------------------------------------------------------*/
void
ComDrv_EnableInterrupt( void )
{
  if ( g_byInterruptMask == 1 )
  {
#if	!USE_SERIAL_INTERFACE
    CPUHost_WriteW569Command( CMD_COM_IRQE );
#else
    CPUHost_SerialReadW569Data( CMD_COM_IRQE | CMD_COM_ID_ST_REG );
#endif
  }
}

/*--------------------------------------------------------------------------*/
short
ComDrv_SendSimpleCommand( BYTE byReg, BYTE byData )
{
  /* select register and disable interrupt */
  ComDrv_DisableInterrupt(  );

  WRITE_W569_DATA( byReg, byData )
      /* enable interrupt */
      ComDrv_EnableInterrupt(  );

  return MW_NO_ERROR;
}

/*--------------------------------------------------------------------------*/
void
ComDrv_EnableInterruptBits( BYTE byBits )
{
  BYTE		byData;

  ComDrv_DisableInterrupt(  );

  WRITE_W569_DATA_OR( CMD_COM_ID_INT_EN_REG, byData, byBits );

  ComDrv_EnableInterrupt(  );
}

/*--------------------------------------------------------------------------*/
void
ComDrv_DisableInterruptBits( BYTE byBits )
{
  BYTE		byData;

  ComDrv_DisableInterrupt(  );

  WRITE_W569_DATA_AND( CMD_COM_ID_INT_EN_REG, byData, ~byBits );

  ComDrv_EnableInterrupt(  );
}

/*--------------------------------------------------------------------------*/
BYTE
ComDrv_ReceiveSimpleData( BYTE byReg )
{
  BYTE		byData;

  /* select Interrupt Status Register and disable interrupt */
  ComDrv_DisableInterrupt(  );

  /* read interrupt status */
  READ_W569_DATA( byReg, byData );

  /* enable interrupt */
  ComDrv_EnableInterrupt(  );

  return byData;
}

/*--------------------------------------------------------------------------*/
static short
ComDrv_ReceiveXBufferData( BYTE * pbyData )
{
#if	NO_TIME_OUT
  while ( ( CPUHost_ReadW569Status(  ) & CMD_STA_X_RDY ) == 0 ) ;
#else
  WORD		i;
  for ( i = 0; i < LOOP_CHECK_COUNT; i++ )
  {
    if ( ( CPUHost_ReadW569Status(  ) & CMD_STA_X_RDY ) != 0 )
      break;
  }
  if ( i == LOOP_CHECK_COUNT )
    return MW_ERROR;
#endif

  READ_W569_DATA( CMD_COM_ID_X_BUF, *pbyData ) return MW_NO_ERROR;;
}

/*--------------------------------------------------------------------------*/
static short
ComDrv_ReceiveYBufferData( BYTE * pbyData )
{
#if	NO_TIME_OUT
  while ( ( CPUHost_ReadW569Status(  ) & CMD_STA_Y_RDY ) == 0 ) ;
#else
  WORD		i;
  for ( i = 0; i < LOOP_CHECK_COUNT; i++ )
  {
    if ( ( CPUHost_ReadW569Status(  ) & CMD_STA_Y_RDY ) != 0 )
      break;
  }
  if ( i == LOOP_CHECK_COUNT )
    return MW_ERROR;
#endif

  READ_W569_DATA( CMD_COM_ID_Y_BUF, *pbyData ) return MW_NO_ERROR;
}

/*--------------------------------------------------------------------------*/
void
ComDrv_Wait( WORD wWaitTime	/* in millisecond */
     )
{
  BYTE		byEnable = 0;
  BYTE		byEnableStatus = 0;

  g_wWaitTime = wWaitTime + 1;

  byEnable = ComDrv_ReceiveSimpleData( CMD_COM_ID_INT_EN_REG );
  ComDrv_DisableInterruptBits( byEnable );
  ComDrv_EnableInterruptBits( IRQ_EN_REG_TAIE );

  /* polling Timer A interrupt */
  while ( g_wWaitTime > 0 )
  {
    byEnableStatus = ComDrv_ReceiveSimpleData( CMD_COM_ID_INT_ST_REG );
    if ( byEnableStatus & IRQ_STA_REG_TAIS )
    {
      ComDrv_SendSimpleCommand( CMD_COM_ID_INT_ST_REG, IRQ_STA_REG_TAIS );
      if ( g_wWaitTime > 0 )
	g_wWaitTime--;
    }
  }

  ComDrv_DisableInterruptBits( IRQ_EN_REG_TAIE );
  byEnable &= ( ~IRQ_EN_REG_TAIE );
  ComDrv_EnableInterruptBits( byEnable );
  ComDrv_SendSimpleCommand( CMD_COM_ID_INT_ST_REG, IRQ_STA_REG_TAIS );
}

/*--------------------------------------------------------------------------*/
static void
ComDrv_VolumeUp( BYTE bySpeakerVolume )
{
  BYTE		byValue, byCount;

  WRITE_W569_DATA_AND_OR( CMD_COM_ID_EQ_VOL_REG, byValue, 0xE0, g_byEQVol );
#if (W56940_64 && (DOUT_LED == 0))
  WRITE_W569_DATA_AND_OR( CMD_COM_ID_HEADPHONE_VOL_L_REG, byValue,
      0xE0, g_byHPLeftVol );
  WRITE_W569_DATA_AND_OR( CMD_COM_ID_HEADPHONE_VOL_R_REG, byValue,
      0xE0, g_byHPRightVol );
#endif

  WRITE_W569_DATA( CMD_COM_ID_EQ_VOL_REG, 0x1F ) ComDrv_Wait( 20 );

  /* Increase speaker volume register to given value */
  byValue = ( SPK_VOL_REG_VDS << 6 ) | 0x01;
  for ( byCount = 1; byCount <= bySpeakerVolume; byCount++, byValue++ )
  {
    WRITE_W569_DATA( CMD_COM_ID_SPK_VOL_REG, byValue );
  }
}

/*--------------------------------------------------------------------------*/
static BYTE
ComDrv_VolumeDown( void )
{
  BYTE		byData;
  BYTE		byRet;

  /* Decrease speaker volume register to 00H one by one */
  READ_W569_DATA( CMD_COM_ID_SPK_VOL_REG, byData );
  byRet = byData & 0x1F;
  while ( ( byData & 0x1F ) > 0 )
  {
    byData--;
    WRITE_W569_DATA( CMD_COM_ID_SPK_VOL_REG, byData );
  }

  return byRet;
}

/*--------------------------------------------------------------------------*/
short
ComDrv_SendFIFOData( BYTE byFIFOReg, const BYTE * pbyData, WORD wDataSize )
{
  WORD		i;

  /* select Sequencer FIFO and disable interrupt */
  ComDrv_DisableInterrupt(  );

  if ( byFIFOReg == CMD_COM_ID_GEN_FIFO )
  {				/* wait for G-FIFO empty */
#if	NO_TIME_OUT
    while ( ( CPUHost_ReadW569Status(  ) & CMD_STA_G_EMP ) == 0x00 ) ;
#else
    for ( i = 0; i < LOOP_CHECK_COUNT; i++ )
    {
      if ( ( CPUHost_ReadW569Status(  ) & CMD_STA_G_EMP ) != 0x00 )
	break;
    }
    if ( i == LOOP_CHECK_COUNT )
      return MW_ERROR_SEND_COMMAND;
#endif
  }

  if ( byFIFOReg == CMD_COM_ID_GEN_FIFO )
    WRITE_W569_DATA( CMD_COM_ID_GEN_FIFO, 0x55 );	// for sync with F/W

  WRITE_W569_BLOCK_DATA( byFIFOReg, pbyData, wDataSize );

  if ( byFIFOReg == CMD_COM_ID_GEN_FIFO )
    WRITE_W569_DATA( CMD_COM_ID_WAKE_UP, WAKE_UP_REG_INT );

  /* enable interrupt */
  ComDrv_EnableInterrupt(  );
  return MW_NO_ERROR;
}

/*--------------------------------------------------------------------------*/
short
ComDrv_PowerUp( void )
{
  WORD		i;

  ComDrv_DisableInterrupt(  );

  /* Set AP2 to 0, Set DP1 to 0 */
  WRITE_W569_DATA( CMD_COM_ID_CLK_CTRL, 0 )
      /* Set DP0 to 0, wake up uC */
      WRITE_W569_DATA( CMD_COM_ID_WAKE_UP, 0 )
      /* Reset W5691 */
      g_byInterruptMask = 0;
  ComDrv_SendSimpleCommand( CMD_COM_ID_FIFO_RT_REG, FIFO_RT_REG_RST );

  /* Wait for W569 initialization done */
#if	NO_TIME_OUT
  while ( ( CPUHost_ReadW569Status(  ) & CMD_STA_BUSY ) != 0 ) ;
#else
  for ( i = 0; i < LOOP_CHECK_COUNT; i++ )
  {
    if ( ( CPUHost_ReadW569Status(  ) & CMD_STA_BUSY ) == 0 )
      break;
  }
  if ( i == LOOP_CHECK_COUNT )
    return MW_ERROR;
#endif

  ComDrv_EnableInterruptBits( IRQ_EN_REG_FIE0 | IRQ_EN_REG_FIE1 |
      IRQ_EN_REG_FIE2 | IRQ_EN_REG_FIE3 );

  g_bPowerUp = TRUE;
  return MW_NO_ERROR;
}

/*--------------------------------------------------------------------------*/
short
ComDrv_PowerUpAnalog( BYTE bySpeakerVolume	/* 0x00 ~ 0x1F */
     )
{
  ComDrv_DisableInterrupt(  );	// added by ctwu1

  /* Make suer speaker volume register is 00H */
  WRITE_W569_DATA( CMD_COM_ID_SPK_VOL_REG, SPK_VOL_REG_VDS << 6 )
      /* Set AP0 and AP1 to 0 */
      WRITE_W569_DATA( CMD_COM_ID_ANALOG_CTRL, ANALOG_CTRL_REG_AP1 | ANALOG_CTRL_REG_AP3 | ANALOG_CTRL_REG_AP4 | ANALOG_CTRL_REG_AP5 | ANALOG_CTRL_REG_AP6 );	/* AP1
																				 * =
																				 * 1,
																				 * AP0
																				 * =
																				 * 0
																				 */
  WRITE_W569_DATA( CMD_COM_ID_ANALOG_CTRL, 0 );	/* AP1 = 0 */

  /* Increment speaker volume register to wanted value one by one */
  ComDrv_VolumeUp( bySpeakerVolume );
  g_bySpeakerVolume = bySpeakerVolume;

  ComDrv_EnableInterrupt(  );

  return MW_NO_ERROR;
}

/*--------------------------------------------------------------------------*/
short
ComDrv_PowerDown( void )
{
  const BYTE	abyStopMode[2] = { 0x03, 0x17 };
  BYTE		byData;
  WORD		j, i;

  ComDrv_DisableInterrupt(  );

  ComDrv_VolumeDown(  );

  /* Set AP1 to 1 */
  WRITE_W569_DATA( CMD_COM_ID_ANALOG_CTRL, ANALOG_CTRL_REG_AP1 );

  /* Set AP0 to 1 */
  WRITE_W569_DATA( CMD_COM_ID_ANALOG_CTRL,
      ANALOG_CTRL_REG_AP0 | ANALOG_CTRL_REG_AP1 |
      ANALOG_CTRL_REG_AP3 | ANALOG_CTRL_REG_AP4 |
      ANALOG_CTRL_REG_AP5 | ANALOG_CTRL_REG_AP6 | ANALOG_CTRL_REG_AP7 );

  /* Let uC enter STOP mode */
  for ( i = 0; i < 5; i++ )
  {
    ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyStopMode, 2 );
    // wait until W5691 enter STOP mode
#if	NO_TIME_OUT
    do
    {
      READ_W569_DATA( CMD_COM_ID_WAKE_UP, byData );
    }
    while ( ( byData & WAKE_UP_REG_DP0 ) == 0 );
    break;
#else
    for ( j = 0; j < LOOP_CHECK_COUNT; j++ )
    {
      READ_W569_DATA( CMD_COM_ID_WAKE_UP, byData );
      if ( ( byData & WAKE_UP_REG_DP0 ) != 0 )
	break;
    }
    if ( j < LOOP_CHECK_COUNT )
      break;
    // enter stop fail,then firmware reset
    WRITE_W569_DATA( CMD_COM_ID_FIFO_RT_REG, FIFO_RT_REG_RST );
    for ( j = 0; j < 20; j++ ) ;
  }

  if ( ( byData & WAKE_UP_REG_DP0 ) == 0 )
    return MW_ERROR;

#endif

  /* Set DP1 to 1, Set AP2 to 1 */
  WRITE_W569_DATA( CMD_COM_ID_CLK_CTRL, CLK_CTRL_REG_DP1 | CLK_CTRL_REG_AP2 );

  ComDrv_EnableInterrupt(  );

  g_bPowerUp = FALSE;
  return MW_NO_ERROR;
}

/*--------------------------------------------------------------------------*/
short
ComDrv_SetMelodyFIFOInterruptPoint( BYTE byIntPoint	/* interrupt point */
     )
{
  BYTE		byRegValue;

  if ( byIntPoint > 7 )
    return MW_ERROR_ARGUMENT;

  /* Select FIFO Status register and disable interrupt */
  ComDrv_DisableInterrupt(  );

  WRITE_W569_DATA_AND_OR( CMD_COM_ID_FIFO_ST_REG, byRegValue, 0xF8,
      byIntPoint & 0x07 );

  /* enable interrupt */
  ComDrv_EnableInterrupt(  );

  return MW_NO_ERROR;
}

#if SUPPORT_SPEECH
/*--------------------------------------------------------------------------*/
static short
ComDrv_SetSpeechFIFOInterruptPoint( BYTE byIntPoint	/* interrupt point */
     )
{
  BYTE		byRegValue;

  if ( byIntPoint > 7 )
    return MW_ERROR_ARGUMENT;

  /* Select FIFO Status register and disable interrupt */
  ComDrv_DisableInterrupt(  );

  WRITE_W569_DATA_AND_OR( CMD_COM_ID_FIFO_ST_REG, byRegValue, 0x8F,
      ( byIntPoint & 0x07 ) << 4 );

  /* enable interrupt */
  ComDrv_EnableInterrupt(  );

  return MW_NO_ERROR;
}
#endif
#if 0
//----------------------------------------------------------------------------
/* Write one byte data to register. */
void
ComDrv_WriteReg( WORD wAddr, BYTE byData )
{
  BYTE		abyData[5] = { 0x01, 0x0, 0x0, 0x0, 0x0 };
  short		i;

  abyData[1] = wAddr & 0xff;
  abyData[2] = ( wAddr >> 8 ) & 0xff;
  abyData[3] = 1;
  abyData[4] = byData;

  ComDrv_SendFIFOData( 0x08, abyData, 5 );
}
#endif
//==============================================================================
/*--------------------------------------------------------------------------*/
static short
SendMelodyData( WORD wSize )
{
  const BYTE	abyTail[3] = { 0xFF, 0xFF, 0x00 };
  const BYTE	abyNOP[3] = { 0xFF, 0x00, 0x00 };
  WORD		wSendSize;

  if ( g_bSendTail )
  {
    if ( g_dwRestMelodyDataSize == 0 )
      ComDrv_SendFIFOData( CMD_COM_ID_SEQ_FIFO, abyTail, 3 );
    else
      ComDrv_SendFIFOData( CMD_COM_ID_SEQ_FIFO, abyNOP, 3 );
    g_bSendTail = FALSE;
    wSize -= 3;
  }

  wSendSize = wSize;
  if ( g_dwRestMelodyDataSize < ( DWORD ) wSize )
    wSendSize = ( WORD ) g_dwRestMelodyDataSize;

  if ( wSendSize > 0 )
  {
    ComDrv_SendFIFOData( CMD_COM_ID_SEQ_FIFO, g_pbyMelodyDataPtr, wSendSize );
    g_pbyMelodyDataPtr += wSendSize;
    g_dwRestMelodyDataSize -= ( DWORD ) wSendSize;
  }

  if ( wSize > wSendSize )
  {
    if ( ( g_byRepeatNum == 0 ) || ( g_byRepeatCount < g_byRepeatNum ) )
    {
      g_pbyMelodyDataPtr = g_pbyMelodyData;
      g_dwRestMelodyDataSize = g_dwMelodyDataSize;
      g_byRepeatCount++;
    }
    wSendSize = wSize - wSendSize;

    if ( g_byFormat == FORMAT_IMF )
    {
      if ( wSendSize >= 3 )
      {
	if ( g_dwRestMelodyDataSize == 0 )
	{
	  ComDrv_SendFIFOData( CMD_COM_ID_SEQ_FIFO, abyTail, 3 );
	  return 0;
	}
	else
	  ComDrv_SendFIFOData( CMD_COM_ID_SEQ_FIFO, abyNOP, 3 );
      }
      else
	g_bSendTail = TRUE;
    }
    else
    {
      if ( g_dwRestMelodyDataSize == 0 )
      {
	ComDrv_SendFIFOData( CMD_COM_ID_SEQ_FIFO, abyTail, 3 );
	ComDrv_DisableInterruptBits( IRQ_EN_REG_SFIE );

	return 0;
      }
    }
  }

  return 1;
}

/*--------------------------------------------------------------------------*/
static short
InterruptSequencerFIFO(  )
{
  short		nRet;
//	printf("Sequencer FIFO Interrupt\n");
  nRet = SendMelodyData( MELODY_FIFO_FILL_SIZE );
  if ( nRet < 0 )
    return MW_ERROR;

  return MW_NO_ERROR;
}

/*--------------------------------------------------------------------------*/
static short
InterruptFirmware1(  )
{
  BYTE		byData;
  BYTE		byChannel;
  short		nRet;

  // printf("InterruptFirmware1\n");

  nRet = ComDrv_ReceiveXBufferData( &byData );
  if ( nRet < 0 )
    return nRet;
//	printf("InterruptFirmware1: byData = 0x%02X\n", byData);
  if ( byData & 0x80 )
  {				// Notify for the sound being audible or not
    switch ( byData )
    {
      case 0x81:		// audible
//			printf("---> Sound is audible <----\n");
	break;
      case 0x82:		// silence
//			printf("---> Sound is silence <----\n");
	break;
    }
  }
  else
  {
    nRet = ComDrv_ReceiveXBufferData( &byChannel );
    if ( nRet < 0 )
      return nRet;
    // end of sequence
    if ( byData & 0x01 )
    {
#if DEBUG_printf
      printf( "---> Melody: end of sequence <----\n" );
#endif
      ComDrv_DisableInterruptBits( IRQ_EN_REG_SFIE );
      if ( g_byPlayStatus & STATUS_SPEECH_PLAYING )
	g_byPlayStatus = STATUS_PLAYING | STATUS_SPEECH_PLAYING;
      else
      {
#if DEBUG_printf
	printf( "Stop and power down.\n" );
#endif
	g_byPlayStatus = STATUS_STOPPED;
#if	SUPPORT_SPEECH
	if ( g_bySpeechPlayStatus == STATUS_STOPPED )
#endif
	  return ComDrv_PowerDown(  );
//				}
      }
    }

#if	SUPPORT_SPEECH
    if ( byData & 0x02 )
    {
#if DEBUG_printf
      printf( "---> Speech: end of sequence (%d) <----\n", byChannel );
#endif
//			bySpeechChannel = byChannel ^ 0x01;
      if ( g_bySpeechPlayStatus & STATUS_SPEECH_PLAYING )
      {
	ComDrv_DisableInterruptBits( IRQ_EN_REG_PFIE );
	g_bySpeechPlayStatus = STATUS_STOPPED;
	if ( g_byPlayStatus == STATUS_STOPPED )
	{
	  // printf("SPEECH END\n");
	  return ComDrv_PowerDown(  );
	}
      }
// else if ((g_PlayInfo[byChannel].byPlayStatus & STATUS_MELODY_PLAYING) &&
//	   (g_PlayInfo[byChannel].byPlayStatus & STATUS_SPEECH_PLAYING))
// {
//   ComDrv_DisableFIFOInterrupt((short)bySpeechChannel);
//   DevDrv_FreeAudioStreamChannel(byChannel, bySpeechChannel);
//   g_PlayInfo[byChannel].byPlayStatus =
//     STATUS_PLAYING | STATUS_MELODY_PLAYING;
// }
    }
#endif
  }

  return MW_NO_ERROR;
}

//*************************************************************//

static short
InterruptFirmware2(  )
{
  BYTE		byType = 0;
  BYTE		abyData[4];
  BYTE		i;
  int		nRet;

  nRet = ComDrv_ReceiveXBufferData( &byType );
  if ( nRet < 0 )
    return nRet;

  if ( byType >= 2 )
    return MW_NO_ERROR;

  for ( i = 0; i < 4; i++ )
  {
    nRet = ComDrv_ReceiveXBufferData( &abyData[i] );
    if ( nRet < 0 )
      return nRet;
  }

  switch ( byType )
  {
    case 0:
#if DEBUG_printf
      printf( "User Evnet : (%d, %d, %d, %d)\n", abyData[0], abyData[1],
	  abyData[2], abyData[3] );
#endif
      break;
    case 1:
      /*
       * if ((abyData[0] == 0xFF) && (abyData[1] == 0xFF))
       * { // Karaoke event CPUHost_W569KaraokeLyricShowNext(); } */
#if DEBUG_printf
      printf( "System Evnet : (%d, %d, %d, %d)\n", abyData[0], abyData[1],
	  abyData[2], abyData[3] );
#endif
      break;
  }
  return MW_NO_ERROR;
}

/*-------------------------------------------------------------*/
#if	SUPPORT_SPEECH
/*--------------------------------------------------------------------------*/
static short
SendSpeechData( WORD wSize )
{
  const BYTE	abyCmdSpeechEOS[] = { 0x03, 0x0F, 0x02 };
  WORD		wSendSize;
  short		nRet = MW_NO_ERROR;

  if ( g_pbyAudioDataPtr == NULL )
    return MW_ERROR;

  wSendSize = wSize;
  if ( g_dwRestAudioDataSize < ( DWORD ) wSize )
    wSendSize = ( WORD ) g_dwRestAudioDataSize;

  if ( wSendSize > 0 )
  {
    nRet = ComDrv_SendFIFOData( CMD_COM_ID_SPE_FIFO,
	g_pbyAudioDataPtr, wSendSize );
    if ( nRet < 0 )
      return nRet;
    // printf("(%d)\n", wSize);
    g_pbyAudioDataPtr += wSendSize;
    g_dwRestAudioDataSize -= ( DWORD ) wSendSize;
  }

  if ( g_dwRestAudioDataSize == 0 )
  {
    if ( ( g_bySpeechRepeatNum == 0 )
	|| ( g_bySpeechRepeatCount < g_bySpeechRepeatNum ) )
    {
      g_bySpeechRepeatCount++;
      g_pbyAudioDataPtr = g_pbyAudioData + 6;
      g_dwRestAudioDataSize = g_dwAudioDataSize - 6;
      if ( ( g_bySpeechRepeatNum == 0 )
	  || ( g_bySpeechRepeatCount < g_bySpeechRepeatNum ) )
	g_dwRestAudioDataSize -= g_bySpeechTailSize;

      return SendSpeechData( wSize - wSendSize );
    }
    else
    {
      nRet = ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyCmdSpeechEOS,
	  sizeof( abyCmdSpeechEOS ) );
      if ( nRet < 0 )
	return nRet;
      ComDrv_DisableInterruptBits( IRQ_EN_REG_PFIE );
      ComDrv_SendSimpleCommand( CMD_COM_ID_INT_ST_REG, IRQ_STA_REG_PFIS );
    }
  }

  return 1;
}

/*--------------------------------------------------------------------------*/
static short
InterruptSpeechFIFO(  )
{
  // printf("Interrupt Speech FIFO\n");
  return SendSpeechData( SPEECH_FIFO_FILL_SIZE );
}

/*--------------------------------------------------------------------------*/
// always use P-FIFO (channel 1)
static short
PlaySpeech( const BYTE * pbyData, DWORD dwDataSize )
{
  BYTE		byStartSpchCmd[8] =
      { 0x03, 0x03, 0x00, 0x00, 0x00, 0x00, 0x7F, 0x40 };
  short		nRet;
  short		nRet2;
  WORD		wSampleRate;
  WORD		wDFA;
  BYTE		byTemp;
  WORD		wFIFOSize = 0;
  WORD		wFIFOFillSize = 0;
//	BYTE byEnable;

  g_pbyAudioData = pbyData;	// +6;
  g_dwAudioDataSize = dwDataSize;	// -6;
  g_pbyAudioDataPtr = pbyData + 6;
  g_dwRestAudioDataSize = dwDataSize - 6;

  g_bySpeechTailSize = 0;
  switch ( g_bySpeechType )
  {
    case eSPEECH_FORMAT_SIGN_8BIT_PCM:
      g_dwAudioDataSize -= 4;
      g_dwRestAudioDataSize -= 4;
      break;
    case eSPEECH_FORMAT_SIGN_16BIT_PCM:
      g_dwAudioDataSize -= 1;
      g_dwRestAudioDataSize -= 1;
      break;
    case eSPEECH_FORMAT_APM:
      g_bySpeechTailSize = 2;
      break;
    default:
      return MW_ERROR;
  }

  if ( g_bySpeechRepeatNum != 1 )
    g_dwRestAudioDataSize -= g_bySpeechTailSize;

  wSampleRate = ( WORD ) pbyData[4] | ( ( WORD ) pbyData[5] << 8 );
//	if (wSampleRate > MAX_SAMPLE_RATE)
//		wSampleRate = MAX_SAMPLE_RATE;

  ComDrv_DisableInterruptBits( IRQ_EN_REG_PFIE );
  ComDrv_SetSpeechFIFOInterruptPoint( ( SPEECH_FIFO_SIZE -
      SPEECH_FIFO_FILL_SIZE ) >> 5 );

  // start speech synthesizer
  byStartSpchCmd[2] = g_bySpeechType;

//	if (g_byPolyphonyMode == 32)
  wSampleRate >>= 1;

  if ( wSampleRate >= ALLOWED_SAMPLE_RATE )
    wDFA = 0x400;
  else
    wDFA = ( WORD ) ( ( ( DWORD ) wSampleRate << 10 ) / ALLOWED_SAMPLE_RATE );

  byStartSpchCmd[3] = ( BYTE ) ( wDFA & 0xFF );
  byStartSpchCmd[4] = ( BYTE ) ( wDFA >> 8 );
  byStartSpchCmd[5] = 0;

  nRet2 = ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO,
      byStartSpchCmd, sizeof( byStartSpchCmd ) );
  if ( nRet2 < 0 )
    return nRet2;

  nRet2 = ComDrv_ReceiveYBufferData( &byTemp );
  if ( nRet2 < 0 )
    return nRet2;

  // clear FIFO
  WRITE_W569_DATA( CMD_COM_ID_INT_ST_REG, IRQ_STA_REG_PFIS );
  ComDrv_SendSimpleCommand( CMD_COM_ID_FIFO_RT_REG, FIFO_RT_REG_PFRT );

  // send part of data into FIFO and then end play protocol to minimize W569 delay
  nRet = SendSpeechData( 32 );
  if ( nRet < 0 )
    return nRet;

  nRet2 = ComDrv_ReceiveYBufferData( &byTemp );
  if ( nRet2 < 0 )
    return nRet2;

  // send rest data into FIFO
  wFIFOSize = SPEECH_FIFO_SIZE - 33;	// don't fill FIFO to full to avoid FIFO full interrupt
  wFIFOFillSize = SPEECH_FIFO_FILL_SIZE;
  while ( wFIFOSize > wFIFOFillSize )
  {
    nRet = SendSpeechData( wFIFOFillSize );
    if ( nRet < 0 )
      return nRet;
    wFIFOSize -= wFIFOFillSize;
  }
  if ( wFIFOSize > 0 )
  {
    nRet = SendSpeechData( wFIFOSize );
    if ( nRet < 0 )
      return nRet;
  }
  if ( nRet == 1 )
    ComDrv_EnableInterruptBits( IRQ_EN_REG_PFIE );

  g_bySpeechPlayStatus = STATUS_PLAYING | STATUS_SPEECH_PLAYING;

  return MW_NO_ERROR;
}
#endif
/*--------------------------------------------------------------------------*/
static short
InterruptHandlerInternal( void )
{
  const BYTE	abyIntType[8] = {
    IRQ_STA_REG_FIS0,		/* F/W interrupt #0 */
    IRQ_STA_REG_FIS1,		/* F/W interrupt #1 */
    IRQ_STA_REG_TBIS,		/* Timer B */
    IRQ_STA_REG_SFIS,		/* Sequencer FIFO */
    IRQ_STA_REG_FIS2,		/* F/W interrupt #2 */
    IRQ_STA_REG_FIS3,		/* F/W interrupt #3 */
    IRQ_STA_REG_TAIS,		/* Timer A */
    IRQ_STA_REG_PFIS		/* Speech FIFO */
  };

  BYTE		byStatus = 0;
  BYTE		byEnable = 0;
  BYTE		byTemp = 0;
  BYTE		byFIFOStatus = 0;
  short		nRet = MW_NO_ERROR;
  BYTE		i;

  byStatus = ComDrv_ReceiveSimpleData( CMD_COM_ID_INT_ST_REG );
  // printf("byStatus = 0x%02X\n", byStatus);

  byEnable = ComDrv_ReceiveSimpleData( CMD_COM_ID_INT_EN_REG );
  // printf("byEnable = 0x%02X\n", byEnable);

  if ( ( byStatus & byEnable ) == 0x00 )
  {
    ComDrv_SendSimpleCommand( CMD_COM_ID_INT_ST_REG, byStatus );
    return MW_NO_ERROR;
  }
  byStatus &= byEnable;

  for ( i = 0; i < 8; i++ )
  {
    byTemp = abyIntType[i];
    if ( byStatus & byTemp )
    {
      switch ( byTemp )
      {
	case IRQ_STA_REG_SFIS:
	  while ( 1 )
	  {
	    ComDrv_SendSimpleCommand( CMD_COM_ID_INT_ST_REG,
		IRQ_STA_REG_SFIS );

	    byFIFOStatus =
		ComDrv_ReceiveSimpleData( CMD_COM_ID_FIFO_STAT_REG );
	    if ( ( byFIFOStatus & FIFO_STAT_REG_SLET ) == 0 )
	      break;

	    if ( ( nRet = InterruptSequencerFIFO(  ) ) < 0 )
	      return nRet;

	    byEnable = ComDrv_ReceiveSimpleData( CMD_COM_ID_INT_EN_REG );
	    if ( ( byEnable & IRQ_EN_REG_SFIE ) == 0 )
	      break;
	  }
	  return MW_NO_ERROR;

#if	SUPPORT_SPEECH
	case IRQ_STA_REG_PFIS:
	  while ( 1 )
	  {
	    ComDrv_SendSimpleCommand( CMD_COM_ID_INT_ST_REG,
		IRQ_STA_REG_PFIS );

	    byFIFOStatus =
		ComDrv_ReceiveSimpleData( CMD_COM_ID_FIFO_STAT_REG );
	    if ( ( byFIFOStatus & FIFO_STAT_REG_PLET ) == 0 )
	      break;

	    if ( ( nRet = InterruptSpeechFIFO(	) ) < 0 )
	      return nRet;

	    byEnable = ComDrv_ReceiveSimpleData( CMD_COM_ID_INT_EN_REG );
	    if ( ( byEnable & IRQ_EN_REG_PFIE ) == 0 )
	      break;
	  }
	  return MW_NO_ERROR;
#endif

	case IRQ_STA_REG_FIS0:
	case IRQ_STA_REG_FIS1:
	case IRQ_STA_REG_FIS2:

/*			case IRQ_STA_REG_FIS3:*/
	  {
	    byFIFOStatus = ComDrv_ReceiveSimpleData( CMD_COM_ID_X_BUF );
	    // printf("%x\n",byFIFOStatus);
	    if ( byFIFOStatus != 0x55 )
	      return MW_NO_ERROR;
	  }
	  break;
      }

      ComDrv_SendSimpleCommand( CMD_COM_ID_INT_ST_REG, byTemp );
      switch ( byTemp )
      {
	case IRQ_STA_REG_FIS0:	/* F/W interrupt #0 */
	case IRQ_STA_REG_TBIS:	/* Timer B */
	case IRQ_STA_REG_TAIS:	/* Timer A */
	case IRQ_STA_REG_FIS2:	/* F/W interrupt #2 */
	  nRet = InterruptFirmware2(  );
//				printf("enter\n");

	case IRQ_STA_REG_FIS3:	/* F/W interrupt #3 */
	  break;
	case IRQ_STA_REG_FIS1:	/* F/W interrupt #1 */
	  nRet = InterruptFirmware1(  );
//				printf("enter\n");
	  break;
	case IRQ_STA_REG_SFIS:	/* Sequencer FIFO */
	  nRet = InterruptSequencerFIFO(  );
	  break;
      }
      if ( nRet < 0 )
	return nRet;
    }
  }

  return MW_NO_ERROR;
}

/*--------------------------------------------------------------------------*
 * Initialize W569
 *--------------------------------------------------------------------------*/
short
W569_Initialize( void )
{
  g_byPlayStatus = STATUS_STOPPED;
#if	SUPPORT_SPEECH
  g_bySpeechPlayStatus = STATUS_STOPPED;
#endif

  g_bySpeakerVolume = 0x1F;
  g_byEQVol = 0x1F;
#if (W56940_64 && (DOUT_LED == 0))
  g_byHPLeftVol = 0x1F;
  g_byHPRightVol = 0x1F;
#endif

  g_bPowerUp = FALSE;

  /* Set PLL */
  ComDrv_SendSimpleCommand( CMD_COM_ID_PLL_ST_REG_I, PLL_ADJUST_N | 0x80 );
  ComDrv_SendSimpleCommand( CMD_COM_ID_PLL_ST_REG_II, PLL_ADJUST_M );
  g_byInterruptMask = 1;

  ComDrv_PowerUp(  );
  ComDrv_PowerDown(  );

  return 0;
}

/*--------------------------------------------------------------------------*
 * Check if W569 is still playing
 *--------------------------------------------------------------------------*/
short
W569_IsPlaying( short nChannelID )
{
  switch ( nChannelID )
  {
#if	SUPPORT_SPEECH
    case 1:			// speech
      return ( g_bySpeechPlayStatus & STATUS_PLAYING ) ? 1 : 0;
    case -1:			// any
      return ( ( g_byPlayStatus & STATUS_PLAYING ) ||
	  ( g_bySpeechPlayStatus & STATUS_PLAYING ) ) ? 1 : 0;
#else
    case -1:			// any
#endif
    case 0:			// melody
      return ( g_byPlayStatus & STATUS_PLAYING ) ? 1 : 0;
    default:
      return 0;
  }
}

/*--------------------------------------------------------------------------*
 * Playback IMF melody data
 *--------------------------------------------------------------------------*/
short
W569_PlayIMF( const BYTE * pbyData, DWORD dwDataSize )
{
  BYTE		abyCmdStartIMF[7] =
      { 0x03, 0x02, 0x00, 0x7F, 0x00, 0x01, 0x00 };
  BYTE		byTemp;
  BYTE		byFirstFIFOSize = MELODY_FIFO_SIZE - MELODY_FIFO_FILL_SIZE;
  short		nRet, nRet2;
  // BYTE byEnable;

  ComDrv_SetMelodyFIFOInterruptPoint( ( MELODY_FIFO_SIZE -
      MELODY_FIFO_FILL_SIZE ) >> 5 );
  ComDrv_SendSimpleCommand( CMD_COM_ID_FIFO_RT_REG, FIFO_RT_REG_SFRT );

  g_pbyMelodyData = pbyData;
  g_dwMelodyDataSize = dwDataSize - 3;
  g_pbyMelodyDataPtr = pbyData;
  g_dwRestMelodyDataSize = dwDataSize - 3;

  ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyCmdStartIMF,
      sizeof( abyCmdStartIMF ) );
  nRet2 = ComDrv_ReceiveYBufferData( &byTemp );
  if ( nRet2 < 0 )
    return nRet2;

  nRet = SendMelodyData( byFirstFIFOSize );
  if ( nRet == 1 )
    nRet = SendMelodyData( MELODY_FIFO_FILL_SIZE );

  g_byPlayStatus = STATUS_PLAYING | STATUS_MELODY_PLAYING;
  nRet2 = ComDrv_ReceiveYBufferData( &byTemp );
  // printf("byTemp=%x\n",byTemp);
  if ( nRet2 < 0 )
    return nRet2;

  if ( nRet == 1 )
    ComDrv_EnableInterruptBits( IRQ_EN_REG_SFIE );

  return MW_NO_ERROR;
}

#if SUPPORT_CMF
/*--------------------------------------------------------------------------*
 * Playback CMF melody data
 *--------------------------------------------------------------------------*/
short
W569_PlayCMF( const BYTE * pbyData, DWORD dwDataSize )
{
  BYTE		abyCmdStartCMF[8] =
      { 0x03, 0x02, 0x03, 0x7F, 0x00, 0x01, 0x00, 0x00 };
  BYTE		byTemp;
  BYTE		byFirstFIFOSize = MELODY_FIFO_SIZE - MELODY_FIFO_FILL_SIZE;
  short		nRet, nRet2;

  ComDrv_SetMelodyFIFOInterruptPoint( ( MELODY_FIFO_SIZE -
      MELODY_FIFO_FILL_SIZE ) >> 5 );
  ComDrv_SendSimpleCommand( CMD_COM_ID_FIFO_RT_REG, FIFO_RT_REG_SFRT );

  g_pbyMelodyData = pbyData + 6;	// skip header
  g_dwMelodyDataSize = dwDataSize - 6;
  g_pbyMelodyDataPtr = pbyData + 6;	// skip header
  g_dwRestMelodyDataSize = dwDataSize - 6;

  abyCmdStartCMF[7] = pbyData[5];
  ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyCmdStartCMF,
      sizeof( abyCmdStartCMF ) );
  nRet2 = ComDrv_ReceiveYBufferData( &byTemp );
  if ( nRet2 < 0 )
    return nRet2;

  nRet = SendMelodyData( byFirstFIFOSize );
  if ( nRet == 1 )
    nRet = SendMelodyData( MELODY_FIFO_FILL_SIZE );

  g_byPlayStatus = STATUS_PLAYING | STATUS_MELODY_PLAYING;
  nRet2 = ComDrv_ReceiveYBufferData( &byTemp );
  if ( nRet2 < 0 )
    return nRet2;

  if ( nRet == 1 )
    ComDrv_EnableInterruptBits( IRQ_EN_REG_SFIE );

  return MW_NO_ERROR;
}
#endif

#if	SUPPORT_LED_CONTROL
/*--------------------------------------------------------------------------*
 * Send LED Setting
 *--------------------------------------------------------------------------*/
static short
SendLEDSetting( void )
{
  short		nRet = 0;
  BYTE		abyData[6] = { 0x03, 0x09, 0, 0, 0, 0 };

  switch ( g_LEDSetting.byMode )
  {
    case 0:			// LED Off
      abyData[3] = 0x00;	// clr = 0, mode = 0
      break;
    case 1:			// Force control
      abyData[3] = 0x0B;	// clr = 1, mode = 3
      break;
    case 2:			// Sync. with Note On/Off event
      abyData[3] = 0x01;	// clr = 0, mode = 1
      break;
    case 3:			// Sync. with LED events
      abyData[3] = 0x0D;	// clr = 1, mode = 5
      break;
  }

  abyData[4] = ( BYTE ) ( g_LEDSetting.wSyncChannel & 0xFF );
  abyData[5] = ( BYTE ) ( g_LEDSetting.wSyncChannel >> 8 );
  abyData[3] |= 0x08;
  nRet = ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyData, 6 );

  if ( nRet < 0 )
    return nRet;

  if ( g_bPowerUp )
  {
    if ( g_LEDSetting.byMode == 0x01 )
      abyData[1] = 0x0B;
    else
      abyData[1] = 0x0C;
    nRet = ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyData, 2 );
  }
  return nRet;
}

/*--------------------------------------------------------------------------*
 * Set LED Control Source
 *--------------------------------------------------------------------------*/
short
W569_LEDControlSource( BYTE bySource )
{
  short		nRet = MW_NO_ERROR;

  if ( bySource > 3 )
    return MW_ERROR_ARGUMENT;

  g_LEDSetting.byMode = bySource;
  return nRet;
}

/*--------------------------------------------------------------------------*
 * Set LED Sync channels for Control source type 2
 *--------------------------------------------------------------------------*/
void
W569_SetLEDSyncChannel( WORD wChannel )
{
  g_LEDSetting.wSyncChannel = wChannel;
}

/*--------------------------------------------------------------------------*
 * Set LED On/Off for Control source type 1
 *--------------------------------------------------------------------------*/
short
W569_SetLEDOnOff( BYTE byRed, BYTE byGreen, BYTE byBlue )
{
  BYTE		abyData[5] = { 0x03, 0x10, 0, 0, 0 };

  if ( g_bPowerUp )
  {
    abyData[2] = byRed;
    abyData[3] = byGreen;
    abyData[4] = byBlue;
    return ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyData, 5 );
  }
  else
  {
    abyData[0] = ComDrv_ReceiveSimpleData( CMD_COM_ID_OP_CTRL ) & 0x01;
    if ( byRed > 0 )
      abyData[0] |= 0x02;
    if ( byGreen > 0 )
      abyData[0] |= 0x04;
    if ( byBlue > 0 )
      abyData[0] |= 0x08;
#if	(W56940_64 && DOUT_LED)
    abyData[1] = ComDrv_ReceiveSimpleData( CMD_COM_ID_ANALOG_CTRL );

    if ( abyData[0] & 0x04 )
      abyData[1] &= ( BYTE ) ~ ( ANALOG_CTRL_REG_AP5 );
    else
      abyData[1] |= ANALOG_CTRL_REG_AP5;
    if ( abyData[0] & 0x08 )
      abyData[1] &= ( BYTE ) ~ ( ANALOG_CTRL_REG_AP6 );
    else
      abyData[1] |= ANALOG_CTRL_REG_AP6;

    ComDrv_SendSimpleCommand( CMD_COM_ID_ANALOG_CTRL, abyData[1] );
#endif
    return ComDrv_SendSimpleCommand( CMD_COM_ID_OP_CTRL, abyData[0] );
  }
}
#endif

#if	SUPPORT_MTR_CONTROL
/*--------------------------------------------------------------------------*
 * Send Motor Setting
 *--------------------------------------------------------------------------*/
static short
SendMotorSetting( void )
{
  short		nRet = 0;
  BYTE		abyData[6] = { 0x03, 0x0A, 0, 0, 0, 0 };

  switch ( g_MotorSetting.byMode )
  {
    case 0:			// LED Off
      abyData[3] = 0x00;	// clr = 0, mode = 0
      break;
    case 1:			// Force control
      abyData[3] = 0x0B;	// clr = 1, mode = 3
      break;
    case 2:			// Sync. with Note On/Off event
      abyData[3] = 0x01;	// clr = 0, mode = 1
      break;
    case 3:			// Sync. with LED events
      abyData[3] = 0x0D;	// clr = 1, mode = 5
      break;
  }

  abyData[4] = ( BYTE ) ( g_MotorSetting.wSyncChannel & 0xFF );
  abyData[5] = ( BYTE ) ( g_MotorSetting.wSyncChannel >> 8 );

  nRet = ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyData, 6 );
  if ( nRet < 0 )
    return nRet;

  if ( g_bPowerUp )
  {
    if ( g_MotorSetting.byMode == 0x01 )
      abyData[1] = 0x0D;
    else
      abyData[1] = 0x0E;
    nRet = ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyData, 2 );
  }
  return nRet;
}

/*--------------------------------------------------------------------------*
 * Set Motor Control Source
 *--------------------------------------------------------------------------*/
short
W569_MotorControlSource( BYTE bySource )
{
  short		nRet = MW_NO_ERROR;

  if ( bySource > 3 )
    return MW_ERROR_ARGUMENT;

  g_MotorSetting.byMode = bySource;
  return nRet;
}

/*--------------------------------------------------------------------------*
 * Set Motor Sync channels for Control source type 2
 *--------------------------------------------------------------------------*/
void
W569_SetMotorSyncChannel( WORD wChannel )
{
  g_MotorSetting.wSyncChannel = wChannel;
}

/*--------------------------------------------------------------------------*
 * Set Motor On/Off for Control source type 1
 *--------------------------------------------------------------------------*/
short
W569_SetMotorOnOff( BYTE byLevel )
{
  BYTE		abyData[3] = { 0x03, 0x11, 0 };
  if ( g_bPowerUp )
  {
    abyData[2] = byLevel;
    return ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyData, 3 );
  }
  else
  {
    abyData[0] = ComDrv_ReceiveSimpleData( CMD_COM_ID_OP_CTRL ) & 0x0E;
    if ( byLevel > 1 )
      abyData[0] |= 0x01;
    return ComDrv_SendSimpleCommand( CMD_COM_ID_OP_CTRL, abyData[0] );
  }
  return 0;
}
#endif

#if 0				// SUPPORT_XMF
/*--------------------------------------------------------------------------*
 * Skip VLQ, move point to the place thay come after VLQ
 *--------------------------------------------------------------------------*/
static short
SkipVLQ( const BYTE * pbyData )
{
  static const BYTE *temp;
  temp = pbyData;
  while ( 1 )
  {
    if ( *temp >= 0x80 )
      temp += 1;
    else
    {
      temp += 1;
      break;
    }
  }
  return temp - pbyData;
}

/*--------------------------------------------------------------------------*
 * Decode XMF
 *--------------------------------------------------------------------------*/
static short
W569_DecodeXMF( const BYTE ** ppbyDataPtr )
{
  static const BYTE abyFileFormatGUID[5][19] = {
    // format sequence defined in MWDefine.h
    // IMF (Immediate Midi)
    {0x80, 0x83, 0x82, 0x83, 0xBC, 0x96, 0xEB, 0xC2, 0x95, 0xB5, 0xBB, 0xA6,
	  0xD8, 0x9A, 0x99, 0xF7, 0xB0, 0x80, 0x9D},
    // APM (Winbond ADPCM)
    {0x80, 0x89, 0xF3, 0xBB, 0x83, 0xBD, 0xA1, 0xC2, 0x9A, 0xF5, 0xB5, 0xAA,
	  0xFE, 0x98, 0x81, 0xCD, 0xFF, 0xD0, 0xBF},
    // CMF (Compressed Midi)
    {0x82, 0xC8, 0x87, 0xE0, 0xF3, 0x8F, 0xC9, 0xF2, 0x8C, 0xE5, 0x8E, 0xA0,
	  0x81, 0xFB, 0xE1, 0x87, 0xE8, 0xD3, 0x94},
    // MBN (Immediate Midi + Speech)
    {0x83, 0xA0, 0xDA, 0xB2, 0xFC, 0xF3, 0xDB, 0xA2, 0xA9, 0x99, 0x98, 0xCC,
	  0xD6, 0x93, 0xD2, 0xA1, 0xA2, 0x98, 0xF5},
    // CMF (Compressed Midi + Speech)
    {0x81, 0xF1, 0x96, 0x9F, 0xD6, 0xDD, 0x8F, 0xC2, 0xB4, 0xA5, 0xAA, 0xD9,
	  0xED, 0xEF, 0xCD, 0xC5, 0x9E, 0x90, 0xD2},
  };
  const BYTE   *pbyTemp = *ppbyDataPtr;

  short		nRet = MW_NO_ERROR;
  BYTE		byValue = 0;
  short		i, j;
//	DWORD dwLEDSync, dwMotorSync;

  // ----check FileHeader----
  // check File ID
/*	if ((pbyTemp[0] != 'X') || (pbyTemp[1] != 'M') || (pbyTemp[2] != 'F') || (pbyTemp[3] != '_'))
	{
		nRet = MW_ERROR;
		return nRet;
	}*/
  pbyTemp += 4;

  // check Version ID
  if ( ( pbyTemp[0] != '1' ) || ( pbyTemp[1] != '.' ) || ( pbyTemp[2] != '0' )
      || ( pbyTemp[3] != '0' ) )
  {
    nRet = MW_ERROR;
    return nRet;
  }
  pbyTemp += 4;

  // read/skip file length
  pbyTemp += SkipVLQ( pbyTemp );

  // read/skip MetaDataTypesTable
  if ( *pbyTemp != 0x02 )	// check Length of Bytes
  {
    nRet = MW_ERROR;
    return nRet;
  }
  pbyTemp += 2;

  // read/skip TreeStart
  pbyTemp += SkipVLQ( pbyTemp );

  // read/skip TreeEnd
  pbyTemp += SkipVLQ( pbyTemp );

  // ----check NodeHeader----
  // read/skip NodeLength
  pbyTemp += SkipVLQ( pbyTemp );

  // check NodeContainItems
  if ( *pbyTemp != 0x00 )	// must be 0 (indicates a file node)
  {
    nRet = MW_ERROR;
    return nRet;
  }
  pbyTemp += 1;

  // read/skip NodeHeaderLength
  pbyTemp += SkipVLQ( pbyTemp );

  // ----check NodeMetaData in NodeHeader----
  // check BytesInLength
  pbyTemp += SkipVLQ( pbyTemp );

  // check Field Specifier
  if ( *pbyTemp != 0x00 )	// must be 0 (indicates Standard)
  {
    nRet = MW_ERROR;
    return nRet;
  }
  pbyTemp += 1;

  // check Field ID
  if ( *pbyTemp != 0x03 )	// must be 3 (indicates Resource Format)
  {
    nRet = MW_ERROR;
    return nRet;
  }
  pbyTemp += 1;

  // check Field Contains
  if ( *pbyTemp != 0x00 )	// must be 0 (indicates Universal)
  {
    nRet = MW_ERROR;
    return nRet;
  }
  pbyTemp += 1;

  // read/skip Length Follow & Visible Binary Data Follow
  pbyTemp += 2;

  // check Format Type ID
  if ( *pbyTemp != 0x03 )	// must be 3 (indicates Non-registered)
  {
    nRet = MW_ERROR;
    return nRet;
  }
  pbyTemp += 1;

  // chekc File Format
  i = 0;
  do
  {
    nRet = MW_NO_ERROR;
    j = 0;
    do
    {
      if ( *pbyTemp == abyFileFormatGUID[i][j] )
	pbyTemp += 1;
      else
      {
	nRet = MW_ERROR;
	pbyTemp -= j;
	break;
      }
      j++;
    }
    while ( j < 19 );
    if ( nRet != MW_ERROR )
      break;
    i++;
  }
  while ( i < 5 );

  if ( nRet == MW_ERROR )
    return nRet;
  g_byFormat = i;

  // chekc File Format
  byValue = *pbyTemp;
  pbyTemp += 1;
  if ( byValue & 0x80 )		// bit 7 means if data bytes follow
  {
    // contain LED/MOTOR Sync Info
    // check if contain
    if ( byValue & 0x10 )	// bit 4 means if LED sync
    {
#if	SUPPORT_LED_CONTROL
      g_LEDSetting.wSyncChannel = ( ( WORD ) ( byValue & 0x03 ) << 14 ) |
	  ( ( WORD ) ( pbyTemp[0] & 0x7F ) << 7 ) |
	  ( ( WORD ) ( pbyTemp[1] & 0x7F ) );
#endif
      pbyTemp += 2;
    }

    if ( byValue & 0x20 )	// bit 5 means if MOTOR sync
    {
#if	SUPPORT_MTR_CONTROL
      g_MotorSetting.wSyncChannel = ( ( WORD ) ( byValue & 0x0C ) << 12 ) |
	  ( ( DWORD ) ( pbyTemp[0] & 0x7F ) << 7 ) |
	  ( ( DWORD ) ( pbyTemp[1] & 0x7F ) );
#endif
      pbyTemp += 2;
    }
  }

  // read/skip NodeUnpackers
  if ( *pbyTemp != 0x01 )	// must be 1 (indicates no Node Unpackers)
  {
    nRet = MW_ERROR;
    return nRet;
  }
  pbyTemp += 1;

  // ----check NodeContain----
  if ( *pbyTemp != 0x01 )	// must be 1 (indicates In-Line Resource)
  {
    nRet = MW_ERROR;
    return nRet;
  }
  pbyTemp += 1;

  *ppbyDataPtr = pbyTemp;
  return nRet;
}
#endif
/*--------------------------------------------------------------------------*
 * Playback melody data
 *
 *	byRepeatCount = 0 for endless repeat
 *--------------------------------------------------------------------------*/
short
W569_Play( const BYTE * pbyData, DWORD dwDataSize, BYTE byRepeatNum )
{
  BYTE		com1[5] = { 0x03, 0x10, 0xFF, 0xFF, 0xFF };
  BYTE		com2[6] = { 0x03, 0x09, 0x40, 0x0B, 0, 0 };
#if	0			// SUPPORT_XMF
  const BYTE   *pbyTemp;
#endif
  short		nRet = MW_NO_ERROR;
  short		nChannel = 0;
  DWORD		dwTag;

  if ( !g_bPowerUp )
  {
    ComDrv_PowerUp(  );
    ComDrv_PowerUpAnalog( g_bySpeakerVolume );
  }

#if	0			// SUPPORT_XMF
  if ( ( pbyData[0] == 'X' ) && ( pbyData[1] == 'M' ) && ( pbyData[2] == 'F' )
      && ( pbyData[3] == '_' ) )
  {
    pbyTemp = pbyData;
    nRet = W569_DecodeXMF( &pbyTemp );
    if ( nRet == MW_ERROR )
      return nRet;

    // calculate total skip header size
    dwDataSize = dwDataSize - ( pbyTemp - pbyData );
    pbyData = pbyTemp;
  }
#endif

  dwTag = ( ( DWORD ) pbyData[0] << 24 ) |
      ( ( DWORD ) pbyData[1] << 16 ) |
      ( ( DWORD ) pbyData[2] << 8 ) | ( ( DWORD ) pbyData[3] );

  switch ( dwTag )
  {
    case 0x00FF107F:		// WIMF
      g_byFormat = FORMAT_IMF;
      // nRet = W569_PlayIMF(pbyData, dwDataSize);
      break;
#if	SUPPORT_CMF
    case 0x57434D32:		// WCM2
      g_byFormat = FORMAT_CMF;
      // nRet = W569_PlayCMF(pbyData, dwDataSize);
      break;
#endif
#if	SUPPORT_SPEECH
    case 0x57504338:		// WPC8
      g_bySpeechType = eSPEECH_FORMAT_SIGN_8BIT_PCM;
      g_byFormat = FORMAT_SPH;
      break;
    case 0x57503136:		// WP16
      g_bySpeechType = eSPEECH_FORMAT_SIGN_16BIT_PCM;
      g_byFormat = FORMAT_SPH;
      break;
    case 0x5741444D:		// WADM
      g_bySpeechType = eSPEECH_FORMAT_APM;
      g_byFormat = FORMAT_SPH;
      break;
#endif
    default:
      return MW_ERROR_FILE;
  };

#if	SUPPORT_SPEECH
  if ( g_byFormat == FORMAT_SPH )
  {
    g_bySpeechRepeatNum = byRepeatNum;
    g_bySpeechRepeatCount = 1;
    nRet = PlaySpeech( pbyData, dwDataSize );
    if ( nRet < 0 )
      return nRet;
    nChannel = 1;
  }
  else
#endif
  {
    g_byRepeatNum = byRepeatNum;
    g_byRepeatCount = 1;
    g_bSendTail = FALSE;

#if	SUPPORT_CMF
    if ( g_byFormat == FORMAT_CMF )
      nRet = W569_PlayCMF( pbyData, dwDataSize );
    else
#endif
      nRet = W569_PlayIMF( pbyData, dwDataSize );
    if ( nRet < 0 )
      return nRet;

#if	SUPPORT_LED_CONTROL
    nRet = SendLEDSetting(  );
    if ( nRet < 0 )
      return nRet;
#endif

#if	SUPPORT_MTR_CONTROL
    nRet = SendMotorSetting(  );
    if ( nRet < 0 )
      return nRet;
#endif
  }

  g_byInterruptMask = 1;
  ComDrv_EnableInterrupt(  );

  return nChannel;
}

/*--------------------------------------------------------------------------*
 * Stop playback
 *--------------------------------------------------------------------------*/
short
W569_Stop( short nChannelID )
{
  BYTE		abyCmdStop[3] = { 0x03, 0x04, 0x01 };
  short		nRet = MW_NO_ERROR;
  // add by Roy
  while ( ( CPUHost_ReadW569Status(  ) & CMD_STA_G_EMP ) == 0x00 )
  {
    W569_InterruptHandler(  );
  }
  // end add by Roy
  if ( nChannelID == 0 )
  {
    if ( g_byPlayStatus == STATUS_STOPPED )
      return MW_NO_ERROR;

    if ( g_byPlayStatus & STATUS_MELODY_PLAYING )
    {
      nRet = ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyCmdStop,
	  sizeof( abyCmdStop ) );
      if ( nRet < 0 )
	return nRet;
      ComDrv_DisableInterruptBits( IRQ_EN_REG_SFIE );
    }
    g_byPlayStatus = STATUS_STOPPED;

#if	SUPPORT_SPEECH
    if ( g_bySpeechPlayStatus == STATUS_STOPPED )
#endif
    {
      // add by Roy
      while ( ( CPUHost_ReadW569Status(  ) & CMD_STA_G_EMP ) == 0x00 )
      {
	W569_InterruptHandler(	);
      }
      // end add by Roy
      ComDrv_PowerDown(  );
    }
  }

#if	SUPPORT_SPEECH
  if ( nChannelID == 1 )
  {
    if ( g_bySpeechPlayStatus == STATUS_STOPPED )
      return MW_NO_ERROR;

    if ( g_bySpeechPlayStatus & STATUS_SPEECH_PLAYING )
    {
      abyCmdStop[2] = 0x02;
      nRet = ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyCmdStop,
	  sizeof( abyCmdStop ) );
      if ( nRet < 0 )
	return nRet;
      ComDrv_DisableInterruptBits( IRQ_EN_REG_PFIE );
    }
    g_bySpeechPlayStatus = STATUS_STOPPED;

    if ( g_byPlayStatus == STATUS_STOPPED )
    {
      // add by Roy
      while ( ( CPUHost_ReadW569Status(  ) & CMD_STA_G_EMP ) == 0x00 )
      {
	W569_InterruptHandler(	);
      }
      // end add by Roy
      ComDrv_PowerDown(  );
    }
  }
#endif
  // printf("status=%x\n",CPUHost_ReadW569Status());
  return MW_NO_ERROR;
}

/*--------------------------------------------------------------------------*
 * Pause playback
 *--------------------------------------------------------------------------*/
short
W569_Pause( short nChannelID )
{
  BYTE		abyCmdPause[] = { 0x03, 0x13, 0x01 };
  short		nRet = MW_NO_ERROR;

  if ( nChannelID == 0 )
  {
    if ( g_byPlayStatus & STATUS_PLAYING )
    {
      if ( g_byPlayStatus & STATUS_MELODY_PLAYING )
      {
	ComDrv_DisableInterrupt(  );
#if	SUPPORT_SPEECH
	if ( !( g_bySpeechPlayStatus & STATUS_PLAYING ) )
#endif
	  g_bySpeakerVolume = ComDrv_VolumeDown(  );
	ComDrv_EnableInterrupt(  );
	nRet = ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyCmdPause,
	    sizeof( abyCmdPause ) );
	g_byPlayStatus = STATUS_PAUSED | STATUS_MELODY_PLAYING;
      }
    }
  }

#if	SUPPORT_SPEECH
  if ( nChannelID == 1 )
  {
    if ( g_bySpeechPlayStatus & STATUS_PLAYING )
    {
      if ( g_bySpeechPlayStatus & STATUS_SPEECH_PLAYING )
      {
	ComDrv_DisableInterrupt(  );
	if ( !( g_byPlayStatus & STATUS_PLAYING ) )
	  g_bySpeakerVolume = ComDrv_VolumeDown(  );
	ComDrv_EnableInterrupt(  );
	abyCmdPause[2] = 0x02;
	nRet = ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyCmdPause,
	    sizeof( abyCmdPause ) );
	g_bySpeechPlayStatus = STATUS_PAUSED | STATUS_SPEECH_PLAYING;
      }
    }
  }
#endif

  return nRet;
}

/*--------------------------------------------------------------------------*
 * Resume playback
 *--------------------------------------------------------------------------*/
short
W569_Resume( short nChannelID )
{
  BYTE		abyCmdResume[] = { 0x03, 0x14, 0x01 };
  short		nRet = MW_NO_ERROR;

  if ( nChannelID == 0 )
  {
    if ( g_byPlayStatus & STATUS_PAUSED )
    {
      if ( g_byPlayStatus & STATUS_MELODY_PLAYING )
      {
	ComDrv_DisableInterrupt(  );
#if	SUPPORT_SPEECH
	if ( !( g_bySpeechPlayStatus & STATUS_PLAYING ) )
#endif
	  ComDrv_VolumeUp( g_bySpeakerVolume );
	ComDrv_EnableInterrupt(  );
	nRet = ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyCmdResume,
	    sizeof( abyCmdResume ) );
	g_byPlayStatus = STATUS_PLAYING | STATUS_MELODY_PLAYING;
      }
    }
  }

#if	SUPPORT_SPEECH
  if ( nChannelID == 1 )
  {
    if ( g_bySpeechPlayStatus & STATUS_PAUSED )
    {
      if ( g_bySpeechPlayStatus & STATUS_SPEECH_PLAYING )
      {
	ComDrv_DisableInterrupt(  );
	if ( !( g_byPlayStatus & STATUS_PLAYING ) )
	  ComDrv_VolumeUp( g_bySpeakerVolume );
	ComDrv_EnableInterrupt(  );
	abyCmdResume[2] = 0x02;
	nRet = ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyCmdResume,
	    sizeof( abyCmdResume ) );
	g_bySpeechPlayStatus = STATUS_PLAYING | STATUS_SPEECH_PLAYING;
      }
    }
  }
#endif

  return nRet;
}

/*--------------------------------------------------------------------------*
 * Set speaker volume
 *--------------------------------------------------------------------------*/
short
W569_SetSpeakerVolume( BYTE byVolume )
{
  if ( byVolume > 0x1F )	// only 5 bits are valid : 0 ~ 1F
    byVolume = 0x1F;
  g_bySpeakerVolume = byVolume;
  return ComDrv_SendSimpleCommand( CMD_COM_ID_SPK_VOL_REG,
      ( ( BYTE ) SPK_VOL_REG_VDS << 6 ) | byVolume );
}

/*--------------------------------------------------------------------------*
 * Set equalizer volume
 *--------------------------------------------------------------------------*/
short
W569_SetEqualizerVolume( BYTE byVolume )
{
  BYTE		byData;

  if ( byVolume > 0x1F )	// only 5 bits are valid : 0 ~ 1F
    byVolume = 0x1F;

  g_byEQVol = byVolume;
  ComDrv_DisableInterrupt(  );

  WRITE_W569_DATA_AND_OR( CMD_COM_ID_EQ_VOL_REG, byData, 0xE0, byVolume );

  ComDrv_EnableInterrupt(  );
  return MW_NO_ERROR;
}

#if	(W56940_64 && (DOUT_LED == 0))
/*--------------------------------------------------------------------------*
 * Set Headphone Left Volume
 *--------------------------------------------------------------------------*/
short
W569_SetHeadphoneLeftVolume( BYTE byVol )
{
  BYTE		byData;

  if ( byVol > 0x1F )		// only 5 bits are valid : 0 ~ 1F
    byVol = 0x1F;

  g_byHPLeftVol = byVol;
  ComDrv_DisableInterrupt(  );

  WRITE_W569_DATA_AND_OR( CMD_COM_ID_HEADPHONE_VOL_L_REG, byData,
      0xE0, byVol );

  ComDrv_EnableInterrupt(  );
  return MW_NO_ERROR;
}

/*--------------------------------------------------------------------------*
 * Set Headphone Right Volume
 *--------------------------------------------------------------------------*/
short
W569_SetHeadphoneRightVolume( BYTE byVol )
{
  BYTE		byData;

  if ( byVol > 0x1F )		// only 5 bits are valid : 0 ~ 1F
    byVol = 0x1F;

  g_byHPRightVol = byVol;
  ComDrv_DisableInterrupt(  );

  WRITE_W569_DATA_AND_OR( CMD_COM_ID_HEADPHONE_VOL_R_REG, byData,
      0xE0, byVol );

  ComDrv_EnableInterrupt(  );
  return MW_NO_ERROR;
}
#endif

/*--------------------------------------------------------------------------*
 * W569 interrupt handler
 *--------------------------------------------------------------------------*/
short
W569_InterruptHandler( void )
{
  short		nRet;

  ComDrv_DisableInterrupt(  );
  g_byInterruptMask = 0;

/*	if (!g_bLEDSettingSent)
		SendLEDSetting();
	if (!g_bMotorSettingSent)
		SendMotorSetting();
*/
  nRet = InterruptHandlerInternal(  );

  g_byInterruptMask = 1;
  ComDrv_EnableInterrupt(  );

  return nRet;
}

/*--------------------------------------------------------------------------*
 * Set Melody or Speech playback volume real-time
 *
 *		byVolume = 0 ~ 127
 *--------------------------------------------------------------------------*/
short
W569_SetSynthesizeVolume( short nChannelID, BYTE byVolume )
{
  BYTE		abyMelodyCmd[4] = { 0x03, 0x18, 0x00, 0x00 };
  BYTE		abySpeechCmd[4] = { 0x03, 0x12, 0x00, 0x00 };

  if ( ( nChannelID < 0 ) || ( nChannelID > 1 ) || ( byVolume > 127 ) )
    return -1;

  if ( nChannelID == 0 )
  {
    abyMelodyCmd[3] = byVolume;
    return ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyMelodyCmd, 4 );
  }
  else if ( nChannelID == 1 )
  {
    abySpeechCmd[3] = byVolume;
    return ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abySpeechCmd, 4 );
  }
  else
    return -2;
}

/*--------------------------------------------------------------------------*
 * Set Melody playback speed real-time
 *
 *		bySpeed =    1	  ~  100   ~   255
 *		  Slowest   Normal   Fastest
 *--------------------------------------------------------------------------*/
short
W569_SetSynthesizeSpeed( short nChannelID, BYTE bySpeed )
{
  BYTE		abyCmd[5] = { 0x03, 0x19, 0x00, 0x00, 0x00 };

  if ( ( nChannelID < 0 ) || ( nChannelID > 1 ) || ( bySpeed == 0 ) )
    return -1;

  if ( nChannelID == 0 )
  {
    abyCmd[3] = bySpeed / 100;
    abyCmd[4] = bySpeed % 100;
    abyCmd[4] = ( BYTE ) ( ( ( WORD ) abyCmd[4] * 255 + 50 ) / 100 );
    return ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyCmd, 5 );
  }
  else
    return -1;
}

/*-----------------------------------------------------------------------------*
 * Set Melody playback pitch shifting value real-time
 *
 *		chPitchShift =	 -24	 ~   0	  ~	24
 *		      -2 octaves   Normal   +2 octaves
 *-----------------------------------------------------------------------------*/
short
W569_SetSynthesizePitchShift( short nChannelID, signed char chPitchShift )
{
#if (W56932 || W56940_64)
  BYTE		abyCmd[4] = { 0x03, 0x1A, 0x00, 0x00 };

  if ( nChannelID == 1 )
    abyCmd[2] = 0x40;
  if ( chPitchShift < 0 )
    abyCmd[3] = ( BYTE ) ( -chPitchShift ) | 0x80;
  else
    abyCmd[3] = ( BYTE ) chPitchShift;
  return ComDrv_SendFIFOData( CMD_COM_ID_GEN_FIFO, abyCmd, 4 );
#endif
}
